# coding: utf-8
import sys
import os
import re
import pickle

import table
from oblovesite import *

# 環境に合わせて以下を書き換えてください。
USER = 'username'
PASSWORD = 'password'
ENCODING = 'euc-jp'
PICKLE_FILENAME = 'evaluation.pickle'


class EvaluationRepository:
    '''
    記事評価の、記事それぞれの履歴についてすべて記録するリポジトリ。
    現在の状態を(サイトから)取得することもできるし、
    過去の履歴を検索して取得することもできる。
    '''
    def __init__(self):
        '''
        コンストラクタ。
        save()で保存された前回の状態を自動的に読み込む。
        '''
        self.site = ObLoveSite("http://www.objectclub.jp:80")
        try:
            f = open(PICKLE_FILENAME, "rb")
            self.results = pickle.load(f)
            f.close()
        except IOError:
            self.results = {}

    def read_current_evaluation(self):
        '''
        現在の記事の評価をサイトから取得して格納する。
        '''
        if not self.site.logged_in:
            self.site.login(USER, PASSWORD)
        result = self.site.get_all_evaluations()
        self.results[result.date] = result

    def save(self):
        '''
        pickleを利用して、リポジトリの中身をすべてファイルに保存する。
        ファイル名はPICKLE_FILENAMEで指定。
        末尾に".bak"をつけたファイルを、バックアップファイルとして作成する。
        '''
        for r in self.results.values():
            r.resolve()


        try:
            f = open("$tmp", "wb")
            pickle.dump(self.results, f, protocol=pickle.HIGHEST_PROTOCOL)
            f.close()

            # バックアップファイルの作成
            try:
                os.remove(PICKLE_FILENAME + ".bak")
            except OSError:
                # バックアップファイルは存在しない; そのまま続行
                pass
            try:
                os.rename(PICKLE_FILENAME, PICKLE_FILENAME + ".bak")
            except OSError:
                # 現在のファイルが存在しない; 問題だけどそのまま続行
                pass
            os.rename("$tmp", PICKLE_FILENAME)
        except:
            os.remove(PICKLE_FILENAME)

    def get_latest(self):
        '''最新(日付が最大)のResultを取り出す'''
        # self.resultsのキーは日付
        keys = self.results.keys()
        keys.sort()
        latest = self.results[keys[-1]]
        return latest

    def get_all_entries(self):
        '''
        すべてのエントリ(すべての記事のすべての評価の履歴)を取得する。
        '''
        entries = []
        for r in self.results.values():
            for e in r.entries:
                entries.append(e)
        return entries

    def get_entry(self, date, id):
        '''
        指定したエントリを取得する。
            date: 指定するエントリの日付
            id: 指定する記事のID
        date,idに該当するものがなければNoneを返す。
        '''
        result = self.results[date]
        for entry in result.entries:
            if entry.id == id:
                return entry
        return None

    def get_result(self, date, exact=True):
        '''
        指定した date の Result を返す。
        exact=Falseの場合は、指定したdateにもっとも近い、
        dateよりも前のResultを返す。
        dateよりも前のResultがなかったら、Noneを返す。
        '''
        if exact:
            return self.results[date]
        else:
            # self.resultsのキーは日付
            keys = self.results.keys()
            keys.sort(reverse=True)
            for k in keys:
                if k <= date:
                    return self.results[k]
            return None

def latest(repo):
    result = repo.get_latest()
    t = table.Table()
    t.add_cell(result.date.strftime("%Y/%m/%d %H:%M:%S"))
    t.add_row()
    t.add_cell(u'ID')
    t.add_cell(u'タイトル')
    t.add_cell(u'良かった')
    t.add_cell(u'普通')
    t.add_cell(u'イマイチ')
    t.add_row()
    for e in result.entries:
        t.add_cell(e.id)
        t.add_cell(e.name.replace(',',''))
        t.add_cell(e.good)
        t.add_cell(e.mid)
        t.add_cell(e.bad)
        t.add_row()
    return t

def latest_diff(repo):
    class EmptyEntry:
        good = ""
        mid = ""
        bad = ""
    class EmptyResult:
        def get_entry(self, id):
            return EmptyEntry()

    result = repo.get_latest()
    today = datetime.datetime.today()
    result_1day = repo.get_result(today - datetime.timedelta(days=1), exact=False) or EmptyResult()
    result_1week = repo.get_result(today - datetime.timedelta(weeks=1), exact=False) or EmptyResult()
    result_1month = repo.get_result(today - datetime.timedelta(days=30), exact=False) or EmptyResult()

    t = table.Table()
    t.add_cell(result.date.strftime("Created at %Y/%m/%d %H:%M:%S"))
    t.add_row()
    t.add_cell(u"ID")
    t.add_cell(u"タイトル")
    t.add_cell(u"")
    t.add_cell(u"最新")
    t.add_cell(u"1日前")
    t.add_cell(u"1週前")
    t.add_cell(u"1月前")
    t.add_row()
    for e in result.entries:
        e_1day = result_1day.get_entry(e.id) or EmptyEntry()
        e_1week = result_1week.get_entry(e.id) or EmptyEntry()
        e_1month = result_1month.get_entry(e.id) or EmptyEntry()
        t.add_cell(e.id)
        t.add_cell(e.name.replace(',',''))
        t.add_cell(u"良かった")
        t.add_cell(e.good)
        t.add_cell(e_1day.good)
        t.add_cell(e_1week.good)
        t.add_cell(e_1month.good)
        t.add_row()
        t.add_cell(e.id)
        t.add_cell(e.name.replace(',',''))
        t.add_cell(u"普通")
        t.add_cell(e.mid)
        t.add_cell(e_1day.mid)
        t.add_cell(e_1week.mid)
        t.add_cell(e_1month.mid)
        t.add_row()
        t.add_cell(e.id)
        t.add_cell(e.name.replace(',',''))
        t.add_cell(u"イマイチ")
        t.add_cell(e.bad)
        t.add_cell(e_1day.bad)
        t.add_cell(e_1week.bad)
        t.add_cell(e_1month.bad)
        t.add_row()
    return t

def all(repo):
    entries = {}
    for e in repo.get_all_entries():
        entries[e.id] = e.name
    entry_ids = list(entries.keys())
    entry_ids.sort()
    dates = {}
    for d in repo.results.keys():
        dates[d] = True
    dates = list(dates.keys())
    dates.sort()

    # 出力
    t = table.Table()
    t.add_cell(datetime.datetime.today().strftime("%Y/%m/%d %H:%M:%S"))
    t.add_row()
    t.add_cell(u"ID")
    t.add_cell(u"タイトル")
    t.add_cell("")
    for d in dates:
        t.add_cell(d.strftime("%Y/%m/%d %H:%M:%S"))
    t.add_row()

    for id in entry_ids:
        r1 = t.get_current_row()
        t.add_row()
        r2 = t.get_current_row()
        t.add_row()
        r3 = t.get_current_row()
        t.add_row()
        r4 = t.get_current_row()
        t.add_row()
        r1.add_cell(id)
        r1.add_cell(entries[id].replace(',',''))
        r1.add_cell(u"合計")
        r2.add_cell(id)
        r2.add_cell(entries[id].replace(',',''))
        r2.add_cell(u"良かった")
        r3.add_cell(id)
        r3.add_cell(entries[id].replace(',',''))
        r3.add_cell(u"普通")
        r4.add_cell(id)
        r4.add_cell(entries[id].replace(',',''))
        r4.add_cell(u"イマイチ")
        for d in dates:
            entry = repo.get_entry(d, id)
            if entry is not None:
                r1.add_cell(entry.good + entry.mid + entry.bad)
                r2.add_cell(entry.good)
                r3.add_cell(entry.mid)
                r4.add_cell(entry.bad)
            else:
                r1.add_cell("")
                r2.add_cell("")
                r3.add_cell("")
                r4.add_cell("")

    for row_idx in range(2, t.row_count()):
        for col_idx in range(3, t.col_count(row_idx) - 1):
            if not t.value(col_idx, row_idx) == t.value(col_idx + 1, row_idx):
                t.get_cell(col_idx + 1, row_idx).color = 'red'
    return t


def main():
    cmd = sys.argv[1]
    repo = EvaluationRepository()

    if cmd == "latest":
        print latest(repo).csv(encoding='utf-8')
    elif cmd == "diff":
        print latest_diff(repo).csv(encoding='utf-8')
    elif cmd == "all":
        print all(repo).csv(encoding='utf-8')
    elif cmd == "current":
        repo.read_current_evaluation()
        repo.save()
    else:
        raise "No cmd found."




if __name__ == '__main__':
    main()


